"""Moves endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import APIResource, MachineVersionDetail, Name, NamedAPIResource, VerboseEffect
from pypokeclient._api.pokemon.abilities import AbilityEffectChange


@dataclass(frozen=True)
class Move:
    id: int
    name: str
    accuracy: int
    effect_chance: int | None
    pp: int
    priority: int
    power: int
    contest_combos: ContestComboSets | None
    contest_type: NamedAPIResource | None
    contest_effect: APIResource | None
    damage_class: NamedAPIResource
    effect_entries: list[VerboseEffect]
    effect_changes: list[AbilityEffectChange]
    learned_by_pokemon: list[NamedAPIResource]
    flavor_text_entries: list[MoveFlavorText]
    generation: NamedAPIResource
    machines: list[MachineVersionDetail]
    meta: MoveMetaData | None
    names: list[Name]
    past_values: list[PastMoveStatValues]
    stat_changes: list[MoveStatChange]
    super_contest_effect: APIResource | None
    target: NamedAPIResource
    type: NamedAPIResource


@dataclass(frozen=True)
class ContestComboSets:
    normal: ContestComboDetail | None
    super: ContestComboDetail | None


@dataclass(frozen=True)
class ContestComboDetail:
    use_before: list[NamedAPIResource] | None
    use_after: list[NamedAPIResource] | None


@dataclass(frozen=True)
class MoveFlavorText:
    flavor_text: str
    language: NamedAPIResource
    version_group: NamedAPIResource


@dataclass(frozen=True)
class MoveMetaData:
    ailment: NamedAPIResource
    category: NamedAPIResource
    min_hits: int | None
    max_hits: int | None
    min_turns: int | None
    max_turns: int | None
    drain: int
    healing: int
    crit_rate: int
    ailment_chance: int
    flinch_chance: int
    stat_chance: int


@dataclass(frozen=True)
class MoveStatChange:
    change: int
    stat: NamedAPIResource


@dataclass(frozen=True)
class PastMoveStatValues:
    accuracy: int
    effect_chance: int
    power: int
    pp: int
    effect_entries: list[VerboseEffect]
    type: NamedAPIResource | None
    version_group: NamedAPIResource



"""Stats endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import APIResource, Name, NamedAPIResource


@dataclass(frozen=True)
class Stat:
    id: int
    name: str
    game_index: int
    is_battle_only: bool
    affecting_moves: MoveStatAffectSets
    affecting_natures: NatureStatAffectSets
    characteristics: list[APIResource]
    move_damage_class: NamedAPIResource | None
    names: list[Name]


@dataclass(frozen=True)
class MoveStatAffectSets:
    increase: list[MoveStatAffect]
    decrease: list[MoveStatAffect]


@dataclass(frozen=True)
class MoveStatAffect:
    change: int
    move: NamedAPIResource


@dataclass(frozen=True)
class NatureStatAffectSets:
    increase: list[NamedAPIResource]
    decrease: list[NamedAPIResource]



"""Location Areas endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource, VersionEncounterDetail


@dataclass(frozen=True)
class LocationArea:
    id: int
    name: str
    game_index: int
    encounter_method_rates: list[EncounterMethodRate]
    location: NamedAPIResource
    names: list[Name]
    pokemon_encounters: list[PokemonEncounter]


@dataclass(frozen=True)
class EncounterMethodRate:
    encounter_method: NamedAPIResource
    version_details: list[EncounterVersionDetails]


@dataclass(frozen=True)
class EncounterVersionDetails:
    rate: int
    version: NamedAPIResource


@dataclass(frozen=True)
class PokemonEncounter:
    pokemon: NamedAPIResource
    version_details: list[VersionEncounterDetail]



"""Characteristics endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Description, NamedAPIResource


@dataclass(frozen=True)
class Characteristic:
    id: int
    gene_modulo: int
    possible_values: list[int]
    highest_stat: NamedAPIResource
    descriptions: list[Description]



"""Move Damage Classes endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Description, Name, NamedAPIResource


@dataclass(frozen=True)
class MoveDamageClass:
    id: int
    name: str
    descriptions: list[Description]
    moves: list[NamedAPIResource]
    names: list[Name]



"""Sprites module."""

from pathlib import Path
from typing import Any

from pydantic import validate_call
from pydantic.dataclasses import dataclass


@dataclass(frozen=True)
class Sprite:
    """A dataclass representing a sprite."""

    url: str
    content: bytes | Any

    @validate_call
    def save(self, path: str | Path) -> None:
        """Save the sprite at the specified path.

        Args:
            path (str | Path): the path where the sprint will be saved.
        """
        with open(path, "wb") as img:
            img.write(self.content)



"""Pal Park Areas endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class PalParkArea:
    id: int
    name: str
    names: list[Name]
    pokemon_encounters: list[PalParkEncounterSpecies]


@dataclass(frozen=True)
class PalParkEncounterSpecies:
    base_score: int
    rate: int
    pokemon_species: NamedAPIResource



"""Natures endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class Nature:
    id: int
    name: str
    decreased_stat: NamedAPIResource | None
    increased_stat: NamedAPIResource | None
    hates_flavor: NamedAPIResource | None
    likes_flavor: NamedAPIResource | None
    pokeathlon_stat_changes: list[NatureStatChange]
    move_battle_style_preferences: list[MoveBattleStylePreference]
    names: list[Name]


@dataclass(frozen=True)
class NatureStatChange:
    max_change: int
    pokeathlon_stat: NamedAPIResource


@dataclass(frozen=True)
class MoveBattleStylePreference:
    low_hp_preference: int
    high_hp_preference: int
    move_battle_style: NamedAPIResource



"""Item Fling Effects endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Effect, NamedAPIResource


@dataclass(frozen=True)
class ItemFlingEffect:
    id: int
    name: str
    effect_entries: list[Effect]
    items: list[NamedAPIResource]



"""Growth Rates endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Description, NamedAPIResource


@dataclass(frozen=True)
class GrowthRate:
    id: int
    name: str
    formula: str
    descriptions: list[Description]
    levels: list[GrowthRateExperienceLevel]
    pokemon_species: list[NamedAPIResource]


@dataclass(frozen=True)
class GrowthRateExperienceLevel:
    level: int
    experience: int



"""Asynchronous client for interacting with PokéAPI."""

import logging
from types import TracebackType

import httpx
from hishel.httpx import AsyncCacheClient
from pydantic import validate_call

from . import _api
from ._api import NAMED_ENDPOINTS, UNNAMED_ENDPOINTS

logger = logging.getLogger(__name__.split(".")[0])


class AsyncClient:
    """Asynchronous version of the client."""

    def __init__(self, http_client: httpx.AsyncClient) -> None:
        """Initializes an `AsyncClient` object.

        Args:
            http_client (httpx.AsyncClient): an `httpx.AsyncClient` used to send request to the API. Please note that
                `hishel.AsyncCacheClient` is a subclass of `httpx.AsyncClient` so you can pass an instance of that
                class if you want to cache the responses.
        """
        if http_client is None:
            self._http_client = httpx.AsyncClient(base_url="https://pokeapi.co/api/v2/")
        else:
            self._http_client = http_client
            if self._http_client.base_url == "":
                self._http_client.base_url = "https://pokeapi.co/api/v2/"

        if isinstance(self._http_client, AsyncCacheClient):
            logger.info(
                f"The asynchronous client is ready and using the cache at "
                f"'{self._http_client._transport.storage.database_path}'."
            )
        else:
            logger.info("The asynchronous client is ready without using any cache.")

    # ===========================================
    # Context manager methods
    # ===========================================
    async def __aenter__(self) -> "AsyncClient":
        """Creates the client and deletes all the expired responses from the cache.

        Returns:
            AsyncClient: an instance of the asynchronous client.
        """
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None = None,
        exc_val: BaseException | None = None,
        exc_tb: TracebackType | None = None,
    ) -> None:
        """Closes the session.

        Args:
            exc_type (type[BaseException] | None): exception type. Defaults to None.
            exc_val (BaseException | None): exception value. Defaults to None.
            exc_tb (TracebackType | None): exception traceback. Defaults to None.
        """
        del exc_type, exc_val, exc_tb
        await self._http_client.aclose()
        logger.info("Closed session for the asynchronous client.")

    # ===========================================
    # HTTP request methods
    # ===========================================
    async def _api_request(self, url: str) -> httpx.Response:
        """Sends an API request.

        Args:
            url (str): the url to which the request will be sent.

        Returns:
            httpx.Response: the non-parsed response from the API.
        """
        try:
            response = await self._http_client.get(url)
            response.raise_for_status()
        except httpx.HTTPError as e:
            raise e

        log_msg = f"[{response.status_code}] Request to {response.url}."
        if response.extensions.get("hishel_from_cache", False):
            log_msg = f"[{response.status_code}] Cached request to {response.url}."

        logger.info(log_msg)
        return response

    async def _get_resource[T](self, endpoint: str, key: int | str, model: type[T]) -> T:
        response = await self._api_request(f"{endpoint}/{key}")
        return model(**response.json())

    async def _get_resources[T](self, endpoint: str, key: int | str, model: type[T]) -> list[T]:
        response = await self._api_request(f"{endpoint}/{key}")
        return [model(**data) for data in response.json()]

    # ===========================================
    # Resource lists
    # ===========================================
    @validate_call
    async def get_resource_list(
        self, endpoint: str, limit: int = 20, offset: int = 0
    ) -> _api.NamedAPIResourceList | _api.APIResourceList | None:
        """Get a list of resource data.

        Args:
            endpoint (str): the endpoint (e.g. "pokemon", "berry", "item").
            limit (int, optional): limits the number of elements to fetch. Defaults to 20.
            offset (int, optional): offset needed to move to next page. Defaults to 0.

        Returns:
            NamedAPIResourceList | APIResourceList | None: the parsed response from the API if the passed endpoint is
                among the list of available endpoints.
        """
        if endpoint in UNNAMED_ENDPOINTS:
            return await self._get_resource(endpoint, f"?limit={limit}&offset={offset}", _api.APIResourceList)
        elif endpoint in NAMED_ENDPOINTS:
            return await self._get_resource(endpoint, f"?limit={limit}&offset={offset}", _api.NamedAPIResourceList)
        else:
            logger.error(f"{endpoint} is not among the list of available endpoints.")
            return None

    # ===========================================
    # Berries group
    # ===========================================
    @validate_call
    async def get_berry(self, key: int | str) -> _api.Berry:
        """Get data about a berry.

        Args:
            key (int | str): id or name of the berry.

        Returns:
            Berry: the parsed response from the API.
        """
        return await self._get_resource("berry", key, _api.Berry)

    @validate_call
    async def get_berry_firmness(self, key: int | str) -> _api.BerryFirmness:
        """Get data about a berry firmness.

        Args:
            key (int | str): id or name of the berry firmness.

        Returns:
            BerryFirmness: the parsed response from the API.
        """
        return await self._get_resource("berry-firmness", key, _api.BerryFirmness)

    @validate_call
    async def get_berry_flavor(self, key: int | str) -> _api.BerryFlavor:
        """Get data about a berry flavor.

        Args:
            key (int | str): id or name of the berry flavor.

        Returns:
            BerryFlavor: the parsed response from the API.
        """
        return await self._get_resource("berry-flavor", key, _api.BerryFlavor)

    # ===========================================
    # Contests group
    # ===========================================
    @validate_call
    async def get_contest_type(self, key: int | str) -> _api.ContestType:
        """Get data about a contest type.

        Args:
            key (int | str): id or name of the contest type.

        Returns:
            ContestType: the parsed response from the API.
        """
        return await self._get_resource("contest-type", key, _api.ContestType)

    @validate_call
    async def get_contest_effect(self, key: int) -> _api.ContestEffect:
        """Get data about a contest effect.

        Args:
            key (int): id of the contest effect.

        Returns:
            ContestEffect: the parsed response from the API.
        """
        return await self._get_resource("contest-effect", key, _api.ContestEffect)

    @validate_call
    async def get_super_contest_effect(self, key: int) -> _api.SuperContestEffect:
        """Get data about a super contest effect.

        Args:
            key (int): id of the super contest effect.

        Returns:
            SuperContestEffect: the parsed response from the API.
        """
        return await self._get_resource("super-contest-effect", key, _api.SuperContestEffect)

    # ===========================================
    # Encounters group
    # ===========================================
    @validate_call
    async def get_encounter_method(self, key: int | str) -> _api.EncounterMethod:
        """Get data about an encounter method.

        Args:
            key (int | str): id or name of the encounter method.

        Returns:
            EncounterMethod: the parsed response from the API.
        """
        return await self._get_resource("encounter-method", key, _api.EncounterMethod)

    @validate_call
    async def get_encounter_condition(self, key: int | str) -> _api.EncounterCondition:
        """Get data about an encounter condition.

        Args:
            key (int | str): id or name of the encounter condition.

        Returns:
            EncounterCondition: the parsed response from the API.
        """
        return await self._get_resource("encounter-condition", key, _api.EncounterCondition)

    @validate_call
    async def get_encounter_condition_value(self, key: int | str) -> _api.EncounterConditionValue:
        """Get data about an encounter condition value.

        Args:
            key (int | str): id or name of the encounter condition.

        Returns:
            EncounterConditionValue: the parsed response from the API.
        """
        return await self._get_resource("encounter-condition-value", key, _api.EncounterConditionValue)

    # ===========================================
    # Evolution group
    # ===========================================
    @validate_call
    async def get_evolution_chain(self, key: int) -> _api.EvolutionChain:
        """Get data about an evolution chain.

        Args:
            key (int): id of the evolution chain.

        Returns:
            EvolutionChain: the parsed response from the API.
        """
        return await self._get_resource("evolution-chain", key, _api.EvolutionChain)

    @validate_call
    async def get_evolution_trigger(self, key: int | str) -> _api.EvolutionTrigger:
        """Get data about an evolution trigger.

        Args:
            key (int | str): id or name of the evolution trigger.

        Returns:
            EvolutionTrigger: the parsed response from the API.
        """
        return await self._get_resource("evolution-trigger", key, _api.EvolutionTrigger)

    # ===========================================
    # Games group
    # ===========================================
    @validate_call
    async def get_generation(self, key: int | str) -> _api.Generation:
        """Get data about a generation.

        Args:
            key (int | str): id or name of the generation.

        Returns:
            Generation: the parsed response from the API.
        """
        return await self._get_resource("generation", key, _api.Generation)

    @validate_call
    async def get_pokedex(self, key: int | str) -> _api.Pokedex:
        """Get data about a Pokédex.

        Args:
            key (int | str): id or name of the Pokédex.

        Returns:
            Pokedex: the parsed response from the API.
        """
        return await self._get_resource("pokedex", key, _api.Pokedex)

    @validate_call
    async def get_version_group(self, key: int | str) -> _api.VersionGroup:
        """Get data about a version group.

        Args:
            key (int | str): id or name of the version group.

        Returns:
            VersionGroup: the parsed response from the API.
        """
        return await self._get_resource("version-group", key, _api.VersionGroup)

    @validate_call
    async def get_version(self, key: int | str) -> _api.Version:
        """Get data about a version.

        Args:
            key (int | str): id or name of the version.

        Returns:
            Version: the parsed response from the API.
        """
        return await self._get_resource("version", key, _api.Version)

    # ===========================================
    # Items group
    # ===========================================
    @validate_call
    async def get_item(self, key: int | str) -> _api.Item:
        """Get data about an item.

        Args:
            key (int | str): id or name of the item.

        Returns:
            Item: the parsed response from the API.
        """
        return await self._get_resource("item", key, _api.Item)

    @validate_call
    async def get_item_attribute(self, key: int | str) -> _api.ItemAttribute:
        """Get data about an item attribute.

        Args:
            key (int | str): id or name of the item attribute.

        Returns:
            ItemAttribute: the parsed response from the API.
        """
        return await self._get_resource("item-attribute", key, _api.ItemAttribute)

    @validate_call
    async def get_item_category(self, key: int | str) -> _api.ItemCategory:
        """Get data about an item category.

        Args:
            key (int | str): id or name of the item category.

        Returns:
            ItemCategory: the parsed response from the API.
        """
        return await self._get_resource("item-category", key, _api.ItemCategory)

    @validate_call
    async def get_item_fling_effect(self, key: int | str) -> _api.ItemFlingEffect:
        """Get data about an item fling effect.

        Args:
            key (int | str): id or name of the item fling effect.

        Returns:
            ItemFlingEffect: the parsed response from the API.
        """
        return await self._get_resource("item-fling-effect", key, _api.ItemFlingEffect)

    @validate_call
    async def get_item_pocket(self, key: int | str) -> _api.ItemPocket:
        """Get data about an item pocket.

        Args:
            key (int | str): id or name of the item pocket.

        Returns:
            ItemPocket: the parsed response from the API.
        """
        return await self._get_resource("item-pocket", key, _api.ItemPocket)

    # ===========================================
    # Locations group
    # ===========================================
    @validate_call
    async def get_location(self, key: int | str) -> _api.Location:
        """Get data about a location.

        Args:
            key (int | str): id or name of the location.

        Returns:
            Location: the parsed response from the API.
        """
        return await self._get_resource("location", key, _api.Location)

    @validate_call
    async def get_location_area(self, key: int | str) -> _api.LocationArea:
        """Get data about a location area.

        Args:
            key (int | str): id or name of the location area.

        Returns:
            LocationArea: the parsed response from the API.
        """
        return await self._get_resource("location-area", key, _api.LocationArea)

    @validate_call
    async def get_pal_park_area(self, key: int | str) -> _api.PalParkArea:
        """Get data about a Pal Park area.

        Args:
            key (int | str): id or name of the Pal Park area.

        Returns:
            PalParkArea: the parsed response from the API.
        """
        return await self._get_resource("pal-park-area", key, _api.PalParkArea)

    @validate_call
    async def get_regions(self, key: int | str) -> _api.Region:
        """Get data about a region.

        Args:
            key (int | str): id or name of the region.

        Returns:
            Region: the parsed response from the API.
        """
        return await self._get_resource("region", key, _api.Region)

    # ===========================================
    # Machines group
    # ===========================================
    @validate_call
    async def get_machine(self, key: int) -> _api.Machine:
        """Get data about a machine.

        Args:
            key (int): id of the machine.

        Returns:
            Machine: the parsed response from the API.
        """
        return await self._get_resource("machine", key, _api.Machine)

    # ===========================================
    # Moves group
    # ===========================================
    @validate_call
    async def get_move(self, key: int | str) -> _api.Move:
        """Get data about a move.

        Args:
            key (int | str): id or name of the move.

        Returns:
            Move: the parsed response from the API.
        """
        return await self._get_resource("move", key, _api.Move)

    @validate_call
    async def get_move_ailment(self, key: int | str) -> _api.MoveAilment:
        """Get data about a move ailment.

        Args:
            key (int | str): id or name of the move ailment.

        Returns:
            MoveAilment: the parsed response from the API.
        """
        return await self._get_resource("move-ailment", key, _api.MoveAilment)

    @validate_call
    async def get_move_battle_style(self, key: int | str) -> _api.MoveBattleStyle:
        """Get data about a move battle style.

        Args:
            key (int | str): id or name of the move battle style.

        Returns:
            MoveBattleStyle: the parsed response from the API.
        """
        return await self._get_resource("move-battle-style", key, _api.MoveBattleStyle)

    @validate_call
    async def get_move_category(self, key: int | str) -> _api.MoveCategory:
        """Get data about a move category.

        Args:
            key (int | str): id or name of the move category.

        Returns:
            MoveCategory: the parsed response from the API.
        """
        return await self._get_resource("move-category", key, _api.MoveCategory)

    @validate_call
    async def get_damage_class(self, key: int | str) -> _api.MoveDamageClass:
        """Get data about a move damage class.

        Args:
            key (int | str): id or name of the move damage class.

        Returns:
            MoveDamageClass: the parsed response from the API.
        """
        return await self._get_resource("move-damage-class", key, _api.MoveDamageClass)

    @validate_call
    async def get_move_learn_method(self, key: int | str) -> _api.MoveLearnMethod:
        """Get data about a move learn method.

        Args:
            key (int | str): id or name of the move learn method.

        Returns:
            MoveLearnMethod: the parsed response from the API.
        """
        return await self._get_resource("move-learn-method", key, _api.MoveLearnMethod)

    @validate_call
    async def get_move_target(self, key: int | str) -> _api.MoveTarget:
        """Get data about a move target.

        Args:
            key (int | str): id or name of the move target.

        Returns:
            MoveTarget: the parsed response from the API.
        """
        return await self._get_resource("move-target", key, _api.MoveTarget)

    # ===========================================
    # Pokémon group
    # ===========================================
    @validate_call
    async def get_ability(self, key: int | str) -> _api.Ability:
        """Get data about an ability.

        Args:
            key (int | str): id or name of the ability.

        Returns:
            Ability: the parsed response from the API.
        """
        return await self._get_resource("ability", key, _api.Ability)

    @validate_call
    async def get_characteristic(self, key: int) -> _api.Characteristic:
        """Get data about a characteristic.

        Args:
            key (int): id of the characteristic.

        Returns:
            Characteristic: the parsed response from the API.
        """
        return await self._get_resource("characteristic", key, _api.Characteristic)

    @validate_call
    async def get_egg_group(self, key: int | str) -> _api.EggGroup:
        """Get data about an egg group.

        Args:
            key (int | str): id or name of the egg group.

        Returns:
            EggGroup: the parsed response from the API.
        """
        return await self._get_resource("egg-group", key, _api.EggGroup)

    @validate_call
    async def get_gender(self, key: int | str) -> _api.Gender:
        """Get data about a gender.

        Args:
            key (int | str): id or name of the gender.

        Returns:
            Gender: the parsed response from the API.
        """
        return await self._get_resource("gender", key, _api.Gender)

    @validate_call
    async def get_growth_rate(self, key: int | str) -> _api.GrowthRate:
        """Get data about a growth rate.

        Args:
            key (int | str): id or name of the growth rate.

        Returns:
            GrowthRate: the parsed response from the API.
        """
        return await self._get_resource("growth-rate", key, _api.GrowthRate)

    @validate_call
    async def get_nature(self, key: int | str) -> _api.Nature:
        """Get data about a nature.

        Args:
            key (int | str): id or name of the nature.

        Returns:
            Nature: the parsed response from the API.
        """
        return await self._get_resource("nature", key, _api.Nature)

    @validate_call
    async def get_pokeathlon_stat(self, key: int | str) -> _api.PokeathlonStat:
        """Get data about a Pokéathlon stat.

        Args:
            key (int | str): id or name of the Pokéathlon stat.

        Returns:
            PokeathlonStat: the parsed response from the API.
        """
        return await self._get_resource("pokeathlon-stat", key, _api.PokeathlonStat)

    @validate_call
    async def get_pokemon(self, key: int | str) -> _api.Pokemon:
        """Get data about a pokémon.

        Args:
            key (int | str): id or name of the pokémon

        Returns:
            Pokemon: the parsed response from the API.
        """
        return await self._get_resource("pokemon", key, _api.Pokemon)

    @validate_call
    async def get_pokemon_location_area(self, key: int | str) -> list[_api.LocationAreaEncounter]:
        """Get data about a Pokémon's encounters in a location area.

        Args:
            key (int | str): id or name of the Pokémon.

        Returns:
            LocationAreaEncounter: the parsed response from the API.
        """
        return await self._get_resources(f"pokemon/{key}/encounters", "", _api.LocationAreaEncounter)

    @validate_call
    async def get_pokemon_color(self, key: int | str) -> _api.PokemonColor:
        """Get data about a Pokémon color.

        Args:
            key (int | str): id or name of the Pokémon color.

        Returns:
            PokemonColor: the parsed response from the API.
        """
        return await self._get_resource("pokemon-color", key, _api.PokemonColor)

    @validate_call
    async def get_pokemon_form(self, key: int | str) -> _api.PokemonForm:
        """Get data about a Pokémon form.

        Args:
            key (int | str): id or name of the Pokémon form.

        Returns:
            PokemonForm: the parsed response from the API.
        """
        return await self._get_resource("pokemon-form", key, _api.PokemonForm)

    @validate_call
    async def get_pokemon_habitat(self, key: int | str) -> _api.PokemonHabitat:
        """Get data about a Pokémon habitat.

        Args:
            key (int | str): id or name of the Pokémon habitat.

        Returns:
            PokemonHabitat: the parsed response from the API.
        """
        return await self._get_resource("pokemon-habitat", key, _api.PokemonHabitat)

    @validate_call
    async def get_pokemon_shape(self, key: int | str) -> _api.PokemonShape:
        """Get data about a Pokémon shape.

        Args:
            key (int | str): id or name of the Pokémon shape.

        Returns:
            PokemonShape: the parsed response from the API.
        """
        return await self._get_resource("pokemon-shape", key, _api.PokemonShape)

    @validate_call
    async def get_pokemon_species(self, key: int | str) -> _api.PokemonSpecies:
        """Get data about a Pokémon species.

        Args:
            key (int | str): id or name of the Pokémon species.

        Returns:
            PokemonSpecies: the parsed response from the API.
        """
        return await self._get_resource("pokemon-species", key, _api.PokemonSpecies)

    @validate_call
    async def get_stat(self, key: int | str) -> _api.Stat:
        """Get data about a stat.

        Args:
            key (int | str): id or name of the stat.

        Returns:
            Stat: the parsed response from the API.
        """
        return await self._get_resource("stat", key, _api.Stat)

    @validate_call
    async def get_type(self, key: int | str) -> _api.Type:
        """Get data about a type.

        Args:
            key (int | str): id or name of the type.

        Returns:
            Type: the parsed response from the API.
        """
        return await self._get_resource("type", key, _api.Type)

    # ===========================================
    # Utility group
    # ===========================================
    @validate_call
    async def get_language(self, key: int | str) -> _api.Language:
        """Get infos about a language.

        Args:
            key (int | str): id or name of the pokémon

        Returns:
            Language: the parsed response from the API.
        """
        return await self._get_resource("language", key, _api.Language)

    # ===========================================
    # Sprites
    # ===========================================
    @validate_call
    async def get_sprite(self, url: str) -> _api.Sprite:
        """Get a sprite from an url.

        Args:
            url (str): the url to the sprite.

        Returns:
            Sprite: a Sprite object useful to save the image.
        """
        response = await self._api_request(url)
        return _api.Sprite(url, response.content)



"""Module that represents the Moves group."""

from .move_ailments import MoveAilment
from .move_battle_styles import MoveBattleStyle
from .move_categories import MoveCategory
from .move_damage_classes import MoveDamageClass
from .move_learn_methods import MoveLearnMethod
from .move_targets import MoveTarget
from .moves import Move

__all__ = ["Move", "MoveAilment", "MoveBattleStyle", "MoveCategory", "MoveDamageClass", "MoveLearnMethod", "MoveTarget"]



"""Abilities endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Effect, Name, NamedAPIResource, VerboseEffect


@dataclass(frozen=True)
class Ability:
    id: int
    name: str
    is_main_series: bool
    generation: NamedAPIResource
    names: list[Name]
    effect_entries: list[VerboseEffect]
    effect_changes: list[AbilityEffectChange]
    flavor_text_entries: list[AbilityFlavorText]
    pokemon: list[AbilityPokemon]


@dataclass(frozen=True)
class AbilityEffectChange:
    effect_entries: list[Effect]
    version_group: NamedAPIResource


@dataclass(frozen=True)
class AbilityFlavorText:
    flavor_text: str
    language: NamedAPIResource
    version_group: NamedAPIResource


@dataclass(frozen=True)
class AbilityPokemon:
    is_hidden: bool
    slot: int
    pokemon: NamedAPIResource



"""Pokemon endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import NamedAPIResource, VersionGameIndex


@dataclass(frozen=True)
class Pokemon:
    id: int
    name: str
    base_experience: int
    height: int
    is_default: bool
    order: int
    weight: int
    abilities: list[PokemonAbility]
    forms: list[NamedAPIResource]
    game_indices: list[VersionGameIndex]
    held_items: list[PokemonHeldItem]
    location_area_encounters: str
    moves: list[PokemonMove]
    past_types: list[PokemonTypePast]
    past_abilities: list[PokemonAbilityPast]
    sprites: PokemonSprite
    cries: PokemonCries
    species: NamedAPIResource
    stats: list[PokemonStat]
    types: list[PokemonType]


@dataclass(frozen=True)
class PokemonAbility:
    is_hidden: bool
    slot: int
    ability: NamedAPIResource | None


@dataclass(frozen=True)
class PokemonType:
    slot: int
    type: NamedAPIResource


@dataclass(frozen=True)
class PokemonTypePast:
    generation: NamedAPIResource
    types: list[PokemonType]


@dataclass(frozen=True)
class PokemonAbilityPast:
    generation: NamedAPIResource
    abilities: list[PokemonAbility]


@dataclass(frozen=True)
class PokemonHeldItem:
    item: NamedAPIResource
    version_details: list[PokemonHeldItemVersion]


@dataclass(frozen=True)
class PokemonHeldItemVersion:
    version: NamedAPIResource
    rarity: int


@dataclass(frozen=True)
class PokemonMove:
    move: NamedAPIResource
    version_group_details: list[PokemonMoveVersion]


@dataclass(frozen=True)
class PokemonMoveVersion:
    move_learn_method: NamedAPIResource
    version_group: NamedAPIResource
    level_learned_at: int
    order: int | None


@dataclass(frozen=True)
class PokemonStat:
    stat: NamedAPIResource
    effort: int
    base_stat: int


@dataclass(frozen=True)
class PokemonSprite:
    front_default: str | None
    front_shiny: str | None
    front_female: str | None
    front_shiny_female: str | None
    back_default: str | None
    back_shiny: str | None
    back_female: str | None
    back_shiny_female: str | None
    other: dict | None = None  # undocumented in PokéAPI docs
    versions: dict | None = None  # undocumented in PokéAPI docs


@dataclass(frozen=True)
class PokemonCries:
    latest: str
    legacy: str | None



"""Contest Effects endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Effect, FlavorText


@dataclass(frozen=True)
class ContestEffect:
    id: int
    appeal: int
    jam: int
    effect_entries: list[Effect]
    flavor_text_entries: list[FlavorText]



"""Module that represents the Locations group."""

from .location_areas import LocationArea
from .locations import Location
from .pal_park_areas import PalParkArea
from .regions import Region

__all__ = ["Location", "LocationArea", "PalParkArea", "Region"]



"""Pokemon Shapes endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class PokemonShape:
    id: int
    name: str
    awesome_names: list[AwesomeName]
    names: list[Name]
    pokemon_species: list[NamedAPIResource]


@dataclass(frozen=True)
class AwesomeName:
    awesome_name: str
    language: NamedAPIResource



"""Module that represents the Contests group."""

from .contest_effects import ContestEffect
from .contest_types import ContestType
from .super_contest_effects import SuperContestEffect

__all__ = ["ContestEffect", "ContestType", "SuperContestEffect"]



"""Pokemon Location Areas endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import NamedAPIResource, VersionEncounterDetail


@dataclass(frozen=True)
class LocationAreaEncounter:
    location_area: NamedAPIResource
    version_details: list[VersionEncounterDetail]



"""Berry Firmnesses endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class BerryFirmness:
    id: int
    name: str
    berries: list[NamedAPIResource]
    names: list[Name]



"""Encounters Methods endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name


@dataclass(frozen=True)
class EncounterMethod:
    id: int
    name: str
    order: int
    names: list[Name]



"""Pokeathlon Stats endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class PokeathlonStat:
    id: int
    name: str
    names: list[Name]
    affecting_natures: NaturePokeathlonStatAffectSets


@dataclass(frozen=True)
class NaturePokeathlonStatAffectSets:
    increase: list[NaturePokeathlonStatAffect]
    decrease: list[NaturePokeathlonStatAffect]


@dataclass(frozen=True)
class NaturePokeathlonStatAffect:
    max_change: int
    nature: NamedAPIResource



"""Genders endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import NamedAPIResource


@dataclass(frozen=True)
class Gender:
    id: int
    name: str
    pokemon_species_details: list[PokemonSpeciesGender]
    required_for_evolution: list[NamedAPIResource]


@dataclass(frozen=True)
class PokemonSpeciesGender:
    rate: int
    pokemon_species: NamedAPIResource



"""Encounter Condition Values endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class EncounterConditionValue:
    id: int
    name: str
    condition: NamedAPIResource
    names: list[Name]



"""Resource Lists/Pagination endpoint."""

from pydantic.dataclasses import dataclass

from .common_models import APIResource, NamedAPIResource


@dataclass(frozen=True)
class NamedAPIResourceList:
    count: int
    next: str | None
    previous: str | None
    results: list[NamedAPIResource]


@dataclass(frozen=True)
class APIResourceList:
    count: int
    next: str | None
    previous: str | None
    results: list[APIResource]



"""Version Groups endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import NamedAPIResource


@dataclass(frozen=True)
class VersionGroup:
    id: int
    name: str
    order: int
    generation: NamedAPIResource
    move_learn_methods: list[NamedAPIResource]
    pokedexes: list[NamedAPIResource]
    regions: list[NamedAPIResource]
    versions: list[NamedAPIResource]



"""Module that represents the Encounters group."""

from .encounter_condition_values import EncounterConditionValue
from .encounter_conditions import EncounterCondition
from .encounter_methods import EncounterMethod

__all__ = ["EncounterCondition", "EncounterConditionValue", "EncounterMethod"]



"""Module that represents the Items group."""

from .item_attributes import ItemAttribute
from .item_categories import ItemCategory
from .item_fling_effects import ItemFlingEffect
from .item_pockets import ItemPocket
from .items import Item

__all__ = ["Item", "ItemAttribute", "ItemCategory", "ItemFlingEffect", "ItemPocket"]



"""Pokemon Species endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import APIResource, Description, FlavorText, Name, NamedAPIResource


@dataclass(frozen=True)
class PokemonSpecies:
    id: int
    name: str
    order: int
    gender_rate: int
    capture_rate: int
    base_happiness: int
    is_baby: bool
    is_legendary: bool
    is_mythical: bool
    hatch_counter: int
    has_gender_differences: bool
    forms_switchable: bool
    growth_rate: NamedAPIResource
    pokedex_numbers: list[PokemonSpeciesDexEntry]
    egg_groups: list[NamedAPIResource]
    color: NamedAPIResource
    shape: NamedAPIResource | None
    evolves_from_species: NamedAPIResource | None
    evolution_chain: APIResource
    habitat: NamedAPIResource | None
    generation: NamedAPIResource
    names: list[Name]
    pal_park_encounters: list[PalParkEncounterArea]
    flavor_text_entries: list[FlavorText]
    form_descriptions: list[Description]
    genera: list[Genus]
    varieties: list[PokemonSpeciesVariety]


@dataclass(frozen=True)
class Genus:
    genus: str
    language: NamedAPIResource


@dataclass(frozen=True)
class PokemonSpeciesDexEntry:
    entry_number: int
    pokedex: NamedAPIResource


@dataclass(frozen=True)
class PalParkEncounterArea:
    base_score: int
    rate: int
    area: NamedAPIResource


@dataclass(frozen=True)
class PokemonSpeciesVariety:
    is_default: bool
    pokemon: NamedAPIResource



"""Contest Types endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import NamedAPIResource


@dataclass(frozen=True)
class ContestType:
    id: int
    name: str
    berry_flavor: NamedAPIResource
    names: list[ContestName]


@dataclass(frozen=True)
class ContestName:
    name: str
    color: str
    language: NamedAPIResource



"""Berries endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import NamedAPIResource


@dataclass(frozen=True)
class Berry:
    id: int
    name: str
    growth_time: int
    max_harvest: int
    natural_gift_power: int
    size: int
    smoothness: int
    soil_dryness: int
    firmness: NamedAPIResource
    flavors: list[BerryFlavorMap]
    item: NamedAPIResource
    natural_gift_type: NamedAPIResource


@dataclass(frozen=True)
class BerryFlavorMap:
    potency: int
    flavor: NamedAPIResource



"""Module that represents the Pokémon group."""

from .abilities import Ability
from .characteristics import Characteristic
from .egg_groups import EggGroup
from .genders import Gender
from .growth_rates import GrowthRate
from .natures import Nature
from .pokeathlon_stats import PokeathlonStat
from .pokemon import Pokemon
from .pokemon_colors import PokemonColor
from .pokemon_forms import PokemonForm
from .pokemon_habitats import PokemonHabitat
from .pokemon_location_areas import LocationAreaEncounter
from .pokemon_shapes import PokemonShape
from .pokemon_species import PokemonSpecies
from .stats import Stat
from .types import Type

__all__ = [
    "Ability",
    "Characteristic",
    "EggGroup",
    "Gender",
    "GrowthRate",
    "LocationAreaEncounter",
    "Nature",
    "PokeathlonStat",
    "Pokemon",
    "PokemonColor",
    "PokemonForm",
    "PokemonHabitat",
    "PokemonShape",
    "PokemonSpecies",
    "Stat",
    "Type",
]



"""Module that represents the api endpoints."""

import re

from .berries import Berry, BerryFirmness, BerryFlavor
from .contests import ContestEffect, ContestType, SuperContestEffect
from .encounters import EncounterCondition, EncounterConditionValue, EncounterMethod
from .evolution import EvolutionChain, EvolutionTrigger
from .games import Generation, Pokedex, Version, VersionGroup
from .items import Item, ItemAttribute, ItemCategory, ItemFlingEffect, ItemPocket
from .languages import Language
from .locations import Location, LocationArea, PalParkArea, Region
from .machines import Machine
from .moves import Move, MoveAilment, MoveBattleStyle, MoveCategory, MoveDamageClass, MoveLearnMethod, MoveTarget
from .pokemon import (
    Ability,
    Characteristic,
    EggGroup,
    Gender,
    GrowthRate,
    LocationAreaEncounter,
    Nature,
    PokeathlonStat,
    Pokemon,
    PokemonColor,
    PokemonForm,
    PokemonHabitat,
    PokemonShape,
    PokemonSpecies,
    Stat,
    Type,
)
from .resource_lists import APIResourceList, NamedAPIResourceList
from .sprites import Sprite

__all__ = [
    "APIResourceList",
    "Ability",
    "Berry",
    "BerryFirmness",
    "BerryFlavor",
    "Characteristic",
    "ContestEffect",
    "ContestType",
    "EggGroup",
    "EncounterCondition",
    "EncounterConditionValue",
    "EncounterMethod",
    "EvolutionChain",
    "EvolutionTrigger",
    "Gender",
    "Generation",
    "GrowthRate",
    "Item",
    "ItemAttribute",
    "ItemCategory",
    "ItemFlingEffect",
    "ItemPocket",
    "Language",
    "Location",
    "LocationArea",
    "LocationAreaEncounter",
    "Machine",
    "Move",
    "MoveAilment",
    "MoveBattleStyle",
    "MoveCategory",
    "MoveDamageClass",
    "MoveLearnMethod",
    "MoveTarget",
    "NamedAPIResourceList",
    "Nature",
    "PalParkArea",
    "PokeathlonStat",
    "Pokedex",
    "Pokemon",
    "PokemonColor",
    "PokemonForm",
    "PokemonHabitat",
    "PokemonShape",
    "PokemonSpecies",
    "Region",
    "Sprite",
    "Stat",
    "SuperContestEffect",
    "Type",
    "Version",
    "VersionGroup",
]

ENDPOINTS = {
    re.sub(r"(\w)([A-Z])", r"\1-\2", endpoint).lower()
    for endpoint in __all__
    if endpoint not in {"APIResourceList", "LocationAreaEncounter", "NamedAPIResourceList", "Sprite"}
}
UNNAMED_ENDPOINTS = {"characteristic", "contest-effect", "evolution-chain", "machine", "super-contest-effect"}
NAMED_ENDPOINTS = ENDPOINTS - UNNAMED_ENDPOINTS



"""Languages endpoint."""

from pydantic.dataclasses import dataclass

from .common_models import Name


@dataclass(frozen=True)
class Language:
    id: int
    name: str
    official: bool
    iso639: str
    iso3166: str
    names: list[Name]



"""Regions endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class Region:
    id: int
    locations: list[NamedAPIResource]
    name: str
    names: list[Name]
    main_generation: NamedAPIResource | None
    pokedexes: list[NamedAPIResource]
    version_groups: list[NamedAPIResource]



"""Evolution Chains endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import NamedAPIResource


@dataclass(frozen=True)
class EvolutionChain:
    id: int
    baby_trigger_item: NamedAPIResource | None
    chain: ChainLink


@dataclass(frozen=True)
class ChainLink:
    is_baby: bool
    species: NamedAPIResource
    evolution_details: list[EvolutionDetail]
    evolves_to: list[ChainLink]


@dataclass(frozen=True)
class EvolutionDetail:
    item: NamedAPIResource | None
    trigger: NamedAPIResource
    gender: int | None
    held_item: NamedAPIResource | None
    known_move: NamedAPIResource | None
    known_move_type: NamedAPIResource | None
    location: NamedAPIResource | None
    min_level: int | None
    min_happiness: int | None
    min_beauty: int | None
    min_affection: int | None
    needs_overworld_rain: bool
    party_species: NamedAPIResource | None
    party_type: NamedAPIResource | None
    relative_physical_stats: int | None
    time_of_day: str
    trade_species: NamedAPIResource | None
    turn_upside_down: bool



"""Super Contest Effects endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import FlavorText, NamedAPIResource


@dataclass(frozen=True)
class SuperContestEffect:
    id: int
    appeal: int
    flavor_text_entries: list[FlavorText]
    moves: list[NamedAPIResource]



"""Berry Flavors endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class BerryFlavor:
    id: int
    name: str
    berries: list[FlavorBerryMap]
    contest_type: NamedAPIResource
    names: list[Name]


@dataclass(frozen=True)
class FlavorBerryMap:
    potency: int
    berry: NamedAPIResource



"""Types endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import GenerationGameIndex, Name, NamedAPIResource


@dataclass(frozen=True)
class Type:
    id: int
    name: str
    damage_relations: TypeRelations
    past_damage_relations: list[TypeRelationsPast]
    game_indices: list[GenerationGameIndex]
    generation: NamedAPIResource
    move_damage_class: NamedAPIResource
    names: list[Name]
    pokemon: list[TypePokemon]
    moves: list[NamedAPIResource]
    sprites: dict | None = None  # undocumented in PokéAPI docs


@dataclass(frozen=True)
class TypePokemon:
    slot: int
    pokemon: NamedAPIResource


@dataclass(frozen=True)
class TypeRelations:
    no_damage_to: list[NamedAPIResource]
    half_damage_to: list[NamedAPIResource]
    double_damage_to: list[NamedAPIResource]
    no_damage_from: list[NamedAPIResource]
    half_damage_from: list[NamedAPIResource]
    double_damage_from: list[NamedAPIResource]


@dataclass(frozen=True)
class TypeRelationsPast:
    generation: NamedAPIResource
    damage_relations: TypeRelations



"""Machines endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import NamedAPIResource


@dataclass(frozen=True)
class Machine:
    id: int
    item: NamedAPIResource
    move: NamedAPIResource
    version_group: NamedAPIResource



"""Item endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import (
    APIResource,
    GenerationGameIndex,
    MachineVersionDetail,
    Name,
    NamedAPIResource,
    VerboseEffect,
    VersionGroupFlavorText,
)


@dataclass(frozen=True)
class Item:
    id: int
    name: str
    cost: int
    fling_power: int | None
    fling_effect: NamedAPIResource | None
    attributes: list[NamedAPIResource]
    category: NamedAPIResource
    effect_entries: list[VerboseEffect]
    flavor_text_entries: list[VersionGroupFlavorText]
    game_indices: list[GenerationGameIndex]
    names: list[Name]
    sprites: ItemSprites | None
    held_by_pokemon: list[ItemHolderPokemon]
    baby_trigger_for: APIResource | None
    machines: list[MachineVersionDetail]


@dataclass(frozen=True)
class ItemSprites:
    default: str | None


@dataclass(frozen=True)
class ItemHolderPokemon:
    pokemon: NamedAPIResource
    version_details: list[ItemHolderPokemonVersionDetail]


@dataclass(frozen=True)
class ItemHolderPokemonVersionDetail:
    rarity: int
    version: NamedAPIResource



"""Locations endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import GenerationGameIndex, Name, NamedAPIResource


@dataclass(frozen=True)
class Location:
    id: int
    name: str
    region: NamedAPIResource | None
    names: list[Name]
    game_indices: list[GenerationGameIndex]
    areas: list[NamedAPIResource]



"""Pokemon Forms endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class PokemonForm:
    id: int
    name: str
    order: int
    form_order: int
    is_default: bool
    is_battle_only: bool
    is_mega: bool
    form_name: str
    pokemon: NamedAPIResource
    types: list[PokemonFormType]
    sprites: PokemonFormSprites
    version_group: NamedAPIResource
    names: list[Name]
    form_names: list[Name]


@dataclass(frozen=True)
class PokemonFormType:
    slot: int
    type: NamedAPIResource


@dataclass(frozen=True)
class PokemonFormSprites:
    front_default: str
    front_shiny: str
    back_default: str
    back_shiny: str



"""Base models shared by the endpoints."""

from __future__ import annotations

from pydantic.dataclasses import dataclass


@dataclass(frozen=True)
class APIResource:
    url: str


@dataclass(frozen=True)
class Description:
    description: str
    language: NamedAPIResource


@dataclass(frozen=True)
class Effect:
    effect: str
    language: NamedAPIResource


@dataclass(frozen=True)
class Encounter:
    min_level: int
    max_level: int
    condition_values: list[NamedAPIResource]
    chance: int
    method: NamedAPIResource


@dataclass(frozen=True)
class FlavorText:
    flavor_text: str
    language: NamedAPIResource
    version: NamedAPIResource | None = None


@dataclass(frozen=True)
class GenerationGameIndex:
    game_index: int
    generation: NamedAPIResource


@dataclass(frozen=True)
class MachineVersionDetail:
    machine: APIResource
    version_group: NamedAPIResource


@dataclass(frozen=True)
class Name:
    name: str
    language: NamedAPIResource


@dataclass(frozen=True)
class NamedAPIResource:
    name: str
    url: str


@dataclass(frozen=True)
class VerboseEffect:
    effect: str
    short_effect: str
    language: NamedAPIResource


@dataclass(frozen=True)
class VersionEncounterDetail:
    version: NamedAPIResource
    max_chance: int
    encounter_details: list[Encounter]


@dataclass(frozen=True)
class VersionGameIndex:
    game_index: int
    version: NamedAPIResource


@dataclass(frozen=True)
class VersionGroupFlavorText:
    text: str
    language: NamedAPIResource
    version_group: NamedAPIResource



"""Move Ailments endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class MoveAilment:
    id: int
    name: str
    moves: list[NamedAPIResource]
    names: list[Name]



"""Module that represents the Berries group."""

from .berries import Berry
from .berry_firmnesses import BerryFirmness
from .berry_flavors import BerryFlavor

__all__ = ["Berry", "BerryFirmness", "BerryFlavor"]



"""Move Categories endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Description, NamedAPIResource


@dataclass(frozen=True)
class MoveCategory:
    id: int
    name: str
    moves: list[NamedAPIResource]
    descriptions: list[Description]



"""Pokedexes endpoint."""

from __future__ import annotations

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Description, Name, NamedAPIResource


@dataclass(frozen=True)
class Pokedex:
    id: int
    name: str
    is_main_series: bool
    descriptions: list[Description]
    names: list[Name]
    pokemon_entries: list[PokemonEntry]
    region: NamedAPIResource | None
    version_groups: list[NamedAPIResource]


@dataclass(frozen=True)
class PokemonEntry:
    entry_number: int
    pokemon_species: NamedAPIResource



"""Egg Groups endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class EggGroup:
    id: int
    name: str
    names: list[Name]
    pokemon_species: list[NamedAPIResource]



"""Generations endpoint."""

from pydantic.dataclasses import dataclass

from pypokeclient._api.common_models import Name, NamedAPIResource


@dataclass(frozen=True)
class Generation:
    id: int
    name: str
    abilities: list[NamedAPIResource]
    names: list[Name]
    main_region: NamedAPIResource
    moves: list[NamedAPIResource]
    pokemon_species: list[NamedAPIResource]
    types: list[NamedAPIResource]
    version_groups: list[NamedAPIResource]



"""Module that represents the Games group."""

from .generations import Generation
from .pokedexes import Pokedex
from .version import Version
from .version_groups import VersionGroup

__all__ = ["Generation", "Pokedex", "Version", "VersionGroup"]
